@@ -1,4 +1,4 @@
-use ruff_python_ast::{self as ast, Arguments, Constant, Expr, Keyword, Stmt};
+use ruff_python_ast::{self as ast, Arguments, Expr, Keyword, Stmt};
 
 use ruff_diagnostics::{Diagnostic, Violation};
 use ruff_macros::{derive_message_formats, violation};
@@ -14,12 +14,12 @@ use crate::registry::Rule;
 /// Checks for abstract classes without abstract methods.
 ///
 /// ## Why is this bad?
-/// Abstract base classes are used to define interfaces. If they have no abstract
-/// methods, they are not useful.
+/// Abstract base classes are used to define interfaces. If an abstract base
+/// class has no abstract methods, you may have forgotten to add an abstract
+/// method to the class or omitted an `@abstractmethod` decorator.
 ///
-/// If the class is not meant to be used as an interface, it should not be an
-/// abstract base class. Remove the `ABC` base class from the class definition,
-/// or add an abstract method to the class.
+/// If the class is _not_ meant to be used as an interface, consider removing
+/// the `ABC` base class from the class definition.
 ///
 /// ## Example
 /// ```python
@@ -56,36 +56,35 @@ impl Violation for AbstractBaseClassWithoutAbstractMethod {
         format!("`{name}` is an abstract base class, but it has no abstract methods")
     }
 }
+
 /// ## What it does
 /// Checks for empty methods in abstract base classes without an abstract
 /// decorator.
 ///
 /// ## Why is this bad?
-/// Empty methods in abstract base classes without an abstract decorator are
-/// indicative of unfinished code or a mistake.
-///
-/// Instead, add an abstract method decorated to indicate that it is abstract,
-/// or implement the method.
+/// Empty methods in abstract base classes without an abstract decorator may be
+/// be indicative of a mistake. If the method is meant to be abstract, add an
+/// `@abstractmethod` decorator to the method.
 ///
 /// ## Example
+///
 /// ```python
 /// from abc import ABC
 ///
 ///
 /// class Foo(ABC):
-///     def method(self):
-///         ...
+///     def method(self): ...
 /// ```
 ///
 /// Use instead:
+///
 /// ```python
 /// from abc import ABC, abstractmethod
 ///
 ///
 /// class Foo(ABC):
 ///     @abstractmethod
-///     def method(self):
-///         ...
+///     def method(self): ...
 /// ```
 ///
 /// ## References
@@ -109,24 +108,26 @@ fn is_abc_class(bases: &[Expr], keywords: &[Keyword], semantic: &SemanticModel)
     keywords.iter().any(|keyword| {
         keyword.arg.as_ref().is_some_and(|arg| arg == "metaclass")
             && semantic
-                .resolve_call_path(&keyword.value)
-                .is_some_and(|call_path| matches!(call_path.as_slice(), ["abc", "ABCMeta"]))
+                .resolve_qualified_name(&keyword.value)
+                .is_some_and(|qualified_name| {
+                    matches!(qualified_name.segments(), ["abc", "ABCMeta"])
+                })
     }) || bases.iter().any(|base| {
         semantic
-            .resolve_call_path(base)
-            .is_some_and(|call_path| matches!(call_path.as_slice(), ["abc", "ABC"]))
+            .resolve_qualified_name(base)
+            .is_some_and(|qualified_name| matches!(qualified_name.segments(), ["abc", "ABC"]))
     })
 }
 
 fn is_empty_body(body: &[Stmt]) -> bool {
     body.iter().all(|stmt| match stmt {
         Stmt::Pass(_) => true,
-        Stmt::Expr(ast::StmtExpr { value, range: _ }) => match value.as_ref() {
-            Expr::Constant(ast::ExprConstant { value, .. }) => {
-                matches!(value, Constant::Str(..) | Constant::Ellipsis)
-            }
-            _ => false,
-        },
+        Stmt::Expr(ast::StmtExpr { value, range: _ }) => {
+            matches!(
+                value.as_ref(),
+                Expr::StringLiteral(_) | Expr::EllipsisLiteral(_)
+            )
+        }
         _ => false,
     })
 }
@@ -154,8 +155,13 @@ pub(crate) fn abstract_base_class(
     let mut has_abstract_method = false;
     for stmt in body {
         // https://github.com/PyCQA/flake8-bugbear/issues/293
-        // Ignore abc's that declares a class attribute that must be set
-        if let Stmt::AnnAssign(_) | Stmt::Assign(_) = stmt {
+        // If an ABC declares an attribute by providing a type annotation
+        // but does not actually assign a value for that attribute,
+        // assume it is intended to be an "abstract attribute"
+        if matches!(
+            stmt,
+            Stmt::AnnAssign(ast::StmtAnnAssign { value: None, .. })
+        ) {
             has_abstract_method = true;
             continue;
         }
