@@ -89,18 +89,19 @@ fn check_msg(checker: &mut Checker, msg: &Expr) {
 /// Check contents of the `extra` argument to logging calls.
 fn check_log_record_attr_clash(checker: &mut Checker, extra: &Keyword) {
     match &extra.value {
-        Expr::Dict(ast::ExprDict { keys, .. }) => {
-            for key in keys {
-                if let Some(key) = &key {
-                    if let Expr::StringLiteral(ast::ExprStringLiteral { value: attr, .. }) = key {
-                        if is_reserved_attr(attr.to_str()) {
-                            checker.diagnostics.push(Diagnostic::new(
-                                LoggingExtraAttrClash(attr.to_string()),
-                                key.range(),
-                            ));
-                        }
-                    }
+        Expr::Dict(dict) => {
+            for invalid_key in dict.iter_keys().filter_map(|key| {
+                let string_key = key?.as_string_literal_expr()?;
+                if is_reserved_attr(string_key.value.to_str()) {
+                    Some(string_key)
+                } else {
+                    None
                 }
+            }) {
+                checker.diagnostics.push(Diagnostic::new(
+                    LoggingExtraAttrClash(invalid_key.value.to_string()),
+                    invalid_key.range(),
+                ));
             }
         }
         Expr::Call(ast::ExprCall {
@@ -108,12 +109,8 @@ fn check_log_record_attr_clash(checker: &mut Checker, extra: &Keyword) {
             arguments: Arguments { keywords, .. },
             ..
         }) => {
-            if checker
-                .semantic()
-                .resolve_qualified_name(func)
-                .is_some_and(|qualified_name| matches!(qualified_name.segments(), ["", "dict"]))
-            {
-                for keyword in keywords.iter() {
+            if checker.semantic().match_builtin_expr(func, "dict") {
+                for keyword in &**keywords {
                     if let Some(attr) = &keyword.arg {
                         if is_reserved_attr(attr) {
                             checker.diagnostics.push(Diagnostic::new(
