@@ -42,7 +42,7 @@ pub(crate) fn unused_annotation(
     for (name, range) in scope.bindings().filter_map(|(name, binding_id)| {
         let binding = checker.semantic().binding(binding_id);
         if binding.kind.is_annotation()
-            && !binding.is_used()
+            && binding.is_unused()
             && !checker.settings.dummy_variable_rgx.is_match(name)
         {
             Some((name.to_string(), binding.range()))
