@@ -4,11 +4,10 @@ use ruff_text_size::{Ranged, TextRange};
 use crate::fix::edits::pad;
 use ruff_diagnostics::{AlwaysFixableViolation, Diagnostic, Edit, Fix};
 use ruff_macros::{derive_message_formats, violation};
-use ruff_python_ast::call_path::compose_call_path;
+use ruff_python_ast::name::{Name, UnqualifiedName};
 use ruff_python_semantic::SemanticModel;
 
 use crate::checkers::ast::Checker;
-use crate::registry::AsRule;
 
 /// ## What it does
 /// Checks for uses of exceptions that alias `OSError`.
@@ -57,89 +56,87 @@ impl AlwaysFixableViolation for OSErrorAlias {
 
 /// Return `true` if an [`Expr`] is an alias of `OSError`.
 fn is_alias(expr: &Expr, semantic: &SemanticModel) -> bool {
-    semantic.resolve_call_path(expr).is_some_and(|call_path| {
-        matches!(
-            call_path.as_slice(),
-            ["", "EnvironmentError" | "IOError" | "WindowsError"]
-                | ["mmap" | "select" | "socket" | "os", "error"]
-        )
-    })
-}
-
-/// Return `true` if an [`Expr`] is `OSError`.
-fn is_os_error(expr: &Expr, semantic: &SemanticModel) -> bool {
     semantic
-        .resolve_call_path(expr)
-        .is_some_and(|call_path| matches!(call_path.as_slice(), ["", "OSError"]))
+        .resolve_qualified_name(expr)
+        .is_some_and(|qualified_name| {
+            matches!(
+                qualified_name.segments(),
+                [
+                    "" | "builtins",
+                    "EnvironmentError" | "IOError" | "WindowsError"
+                ] | ["mmap" | "select" | "socket" | "os", "error"]
+            )
+        })
 }
 
 /// Create a [`Diagnostic`] for a single target, like an [`Expr::Name`].
 fn atom_diagnostic(checker: &mut Checker, target: &Expr) {
     let mut diagnostic = Diagnostic::new(
         OSErrorAlias {
-            name: compose_call_path(target),
+            name: UnqualifiedName::from_expr(target).map(|name| name.to_string()),
         },
         target.range(),
     );
-    if checker.patch(diagnostic.kind.rule()) {
-        if checker.semantic().is_builtin("OSError") {
-            diagnostic.set_fix(Fix::automatic(Edit::range_replacement(
-                "OSError".to_string(),
-                target.range(),
-            )));
-        }
-    }
+    diagnostic.try_set_fix(|| {
+        let (import_edit, binding) = checker.importer().get_or_import_builtin_symbol(
+            "OSError",
+            target.start(),
+            checker.semantic(),
+        )?;
+        Ok(Fix::safe_edits(
+            Edit::range_replacement(binding, target.range()),
+            import_edit,
+        ))
+    });
     checker.diagnostics.push(diagnostic);
 }
 
 /// Create a [`Diagnostic`] for a tuple of expressions.
 fn tuple_diagnostic(checker: &mut Checker, tuple: &ast::ExprTuple, aliases: &[&Expr]) {
     let mut diagnostic = Diagnostic::new(OSErrorAlias { name: None }, tuple.range());
-    if checker.patch(diagnostic.kind.rule()) {
-        if checker.semantic().is_builtin("OSError") {
-            // Filter out any `OSErrors` aliases.
-            let mut remaining: Vec<Expr> = tuple
-                .elts
-                .iter()
-                .filter_map(|elt| {
-                    if aliases.contains(&elt) {
-                        None
-                    } else {
-                        Some(elt.clone())
-                    }
-                })
-                .collect();
+    let semantic = checker.semantic();
+    if semantic.has_builtin_binding("OSError") {
+        // Filter out any `OSErrors` aliases.
+        let mut remaining: Vec<Expr> = tuple
+            .iter()
+            .filter_map(|element| {
+                if aliases.contains(&element) {
+                    None
+                } else {
+                    Some(element.clone())
+                }
+            })
+            .collect();
 
-            // If `OSError` itself isn't already in the tuple, add it.
-            if tuple
-                .elts
-                .iter()
-                .all(|elt| !is_os_error(elt, checker.semantic()))
-            {
-                let node = ast::ExprName {
-                    id: "OSError".into(),
-                    ctx: ExprContext::Load,
-                    range: TextRange::default(),
-                };
-                remaining.insert(0, node.into());
-            }
-
-            let content = if remaining.len() == 1 {
-                "OSError".to_string()
-            } else {
-                let node = ast::ExprTuple {
-                    elts: remaining,
-                    ctx: ExprContext::Load,
-                    range: TextRange::default(),
-                };
-                format!("({})", checker.generator().expr(&node.into()))
+        // If `OSError` itself isn't already in the tuple, add it.
+        if tuple
+            .iter()
+            .all(|elem| !semantic.match_builtin_expr(elem, "OSError"))
+        {
+            let node = ast::ExprName {
+                id: Name::new_static("OSError"),
+                ctx: ExprContext::Load,
+                range: TextRange::default(),
             };
-
-            diagnostic.set_fix(Fix::automatic(Edit::range_replacement(
-                pad(content, tuple.range(), checker.locator()),
-                tuple.range(),
-            )));
+            remaining.insert(0, node.into());
         }
+
+        let content = if remaining.len() == 1 {
+            "OSError".to_string()
+        } else {
+            let node = ast::ExprTuple {
+                elts: remaining,
+                ctx: ExprContext::Load,
+                range: TextRange::default(),
+                parenthesized: true,
+            };
+            format!("({})", checker.generator().expr(&node.into()))
+        };
+
+        diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
+            pad(content, tuple.range(), checker.locator()),
+            tuple.range(),
+        )));
     }
     checker.diagnostics.push(diagnostic);
 }
@@ -160,9 +157,9 @@ pub(crate) fn os_error_alias_handlers(checker: &mut Checker, handlers: &[ExceptH
             Expr::Tuple(tuple) => {
                 // List of aliases to replace with `OSError`.
                 let mut aliases: Vec<&Expr> = vec![];
-                for elt in &tuple.elts {
-                    if is_alias(elt, checker.semantic()) {
-                        aliases.push(elt);
+                for element in tuple {
+                    if is_alias(element, checker.semantic()) {
+                        aliases.push(element);
                     }
                 }
                 if !aliases.is_empty() {
