@@ -5,8 +5,8 @@ use anyhow::Result;
 use ruff_diagnostics::{Diagnostic, Edit, Fix, FixAvailability, Violation};
 use ruff_macros::{derive_message_formats, violation};
 
+use ruff_python_ast::name::Name;
 use ruff_python_ast::{self as ast, Expr, Operator, ParameterWithDefault, Parameters};
-use ruff_python_parser::typing::parse_type_annotation;
 use ruff_text_size::{Ranged, TextRange};
 
 use crate::checkers::ast::Checker;
@@ -145,7 +145,7 @@ fn generate_fix(checker: &Checker, conversion_type: ConversionType, expr: &Expr)
             let new_expr = Expr::Subscript(ast::ExprSubscript {
                 range: TextRange::default(),
                 value: Box::new(Expr::Name(ast::ExprName {
-                    id: binding,
+                    id: Name::new(binding),
                     ctx: ast::ExprContext::Store,
                     range: TextRange::default(),
                 })),
@@ -177,15 +177,12 @@ pub(crate) fn implicit_optional(checker: &mut Checker, parameters: &Parameters)
             continue;
         };
 
-        if let Expr::StringLiteral(ast::ExprStringLiteral { range, value }) = annotation.as_ref() {
+        if let Expr::StringLiteral(string_expr) = annotation.as_ref() {
             // Quoted annotation.
-            if let Ok((annotation, kind)) =
-                parse_type_annotation(value.to_str(), *range, checker.locator().contents())
-            {
+            if let Some(parsed_annotation) = checker.parse_type_annotation(string_expr) {
                 let Some(expr) = type_hint_explicitly_allows_none(
-                    &annotation,
-                    checker.semantic(),
-                    checker.locator(),
+                    parsed_annotation.expression(),
+                    checker,
                     checker.settings.target_version.minor(),
                 ) else {
                     continue;
@@ -194,7 +191,7 @@ pub(crate) fn implicit_optional(checker: &mut Checker, parameters: &Parameters)
 
                 let mut diagnostic =
                     Diagnostic::new(ImplicitOptional { conversion_type }, expr.range());
-                if kind.is_simple() {
+                if parsed_annotation.kind().is_simple() {
                     diagnostic.try_set_fix(|| generate_fix(checker, conversion_type, expr));
                 }
                 checker.diagnostics.push(diagnostic);
@@ -203,8 +200,7 @@ pub(crate) fn implicit_optional(checker: &mut Checker, parameters: &Parameters)
             // Unquoted annotation.
             let Some(expr) = type_hint_explicitly_allows_none(
                 annotation,
-                checker.semantic(),
-                checker.locator(),
+                checker,
                 checker.settings.target_version.minor(),
             ) else {
                 continue;
