@@ -24,10 +24,13 @@ use crate::importer::ImportRequest;
 /// quadratic complexity. The following methods are all linear in the number of
 /// lists:
 ///
-/// - `functools.reduce(operator.iconcat, lists, [])`
+/// - `functools.reduce(operator.iadd, lists, [])`
 /// - `list(itertools.chain.from_iterable(lists))`
 /// - `[item for sublist in lists for item in sublist]`
 ///
+/// When fixing relevant violations, Ruff defaults to the `functools.reduce`
+/// form, which outperforms the other methods in [microbenchmarks].
+///
 /// ## Example
 /// ```python
 /// lists = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
@@ -41,12 +44,14 @@ use crate::importer::ImportRequest;
 ///
 ///
 /// lists = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
-/// functools.reduce(operator.iconcat, lists, [])
+/// functools.reduce(operator.iadd, lists, [])
 /// ```
 ///
 /// ## References
 /// - [_How Not to Flatten a List of Lists in Python_](https://mathieularose.com/how-not-to-flatten-a-list-of-lists-in-python)
 /// - [_How do I make a flat list out of a list of lists?_](https://stackoverflow.com/questions/952914/how-do-i-make-a-flat-list-out-of-a-list-of-lists/953097#953097)
+///
+/// [microbenchmarks]: https://github.com/astral-sh/ruff/issues/5073#issuecomment-1591836349
 #[violation]
 pub struct QuadraticListSummation;
 
@@ -69,15 +74,17 @@ pub(crate) fn quadratic_list_summation(checker: &mut Checker, call: &ast::ExprCa
         range,
     } = call;
 
-    if !func_is_builtin(func, "sum", checker.semantic()) {
-        return;
-    }
-
-    if !start_is_empty_list(arguments, checker.semantic()) {
+    let Some(iterable) = arguments.args.first() else {
         return;
     };
 
-    let Some(iterable) = arguments.args.first() else {
+    let semantic = checker.semantic();
+
+    if !semantic.match_builtin_expr(func, "sum") {
+        return;
+    }
+
+    if !start_is_empty_list(arguments, semantic) {
         return;
     };
 
@@ -104,7 +111,7 @@ fn convert_to_reduce(iterable: &Expr, call: &ast::ExprCall, checker: &Checker) -
         parenthesized_range(
             iterable.into(),
             call.arguments.as_any_node_ref(),
-            checker.indexer().comment_ranges(),
+            checker.comment_ranges(),
             checker.locator().contents(),
         )
         .unwrap_or(iterable.range()),
@@ -119,14 +126,6 @@ fn convert_to_reduce(iterable: &Expr, call: &ast::ExprCall, checker: &Checker) -
     ))
 }
 
-/// Check if a function is a builtin with a given name.
-fn func_is_builtin(func: &Expr, name: &str, semantic: &SemanticModel) -> bool {
-    let Expr::Name(ast::ExprName { id, .. }) = func else {
-        return false;
-    };
-    id == name && semantic.is_builtin(id)
-}
-
 /// Returns `true` if the `start` argument to a `sum()` call is an empty list.
 fn start_is_empty_list(arguments: &Arguments, semantic: &SemanticModel) -> bool {
     let Some(start_arg) = arguments.find_argument("start", 1) else {
@@ -136,8 +135,8 @@ fn start_is_empty_list(arguments: &Arguments, semantic: &SemanticModel) -> bool
     match start_arg {
         Expr::Call(ast::ExprCall {
             func, arguments, ..
-        }) => arguments.is_empty() && func_is_builtin(func, "list", semantic),
-        Expr::List(ast::ExprList { elts, ctx, .. }) => elts.is_empty() && ctx.is_load(),
+        }) => arguments.is_empty() && semantic.match_builtin_expr(func, "list"),
+        Expr::List(list) => list.is_empty() && list.ctx.is_load(),
         _ => false,
     }
 }
