@@ -8,10 +8,10 @@ use syn::{
     Ident, ItemFn, LitStr, Pat, Path, Stmt, Token,
 };
 
-use crate::rule_code_prefix::{get_prefix_ident, if_all_same};
+use crate::rule_code_prefix::{get_prefix_ident, intersection_all};
 
 /// A rule entry in the big match statement such a
-/// `(Pycodestyle, "E112") => (RuleGroup::Nursery, rules::pycodestyle::rules::logical_lines::NoIndentedBlock),`
+/// `(Pycodestyle, "E112") => (RuleGroup::Preview, rules::pycodestyle::rules::logical_lines::NoIndentedBlock),`
 #[derive(Clone)]
 struct Rule {
     /// The actual name of the rule, e.g., `NoIndentedBlock`.
@@ -20,7 +20,7 @@ struct Rule {
     linter: Ident,
     /// The code associated with the rule, e.g., `"E112"`.
     code: LitStr,
-    /// The rule group identifier, e.g., `RuleGroup::Nursery`.
+    /// The rule group identifier, e.g., `RuleGroup::Preview`.
     group: Path,
     /// The path to the struct implementing the rule, e.g.
     /// `rules::pycodestyle::rules::logical_lines::NoIndentedBlock`
@@ -142,12 +142,13 @@ pub(crate) fn map_codes(func: &ItemFn) -> syn::Result<TokenStream> {
 
         for (prefix, rules) in &rules_by_prefix {
             let prefix_ident = get_prefix_ident(prefix);
-            let attr = match if_all_same(rules.iter().map(|(.., attrs)| attrs)) {
-                Some(attr) => quote!(#(#attr)*),
-                None => quote!(),
+            let attrs = intersection_all(rules.iter().map(|(.., attrs)| attrs.as_slice()));
+            let attrs = match attrs.as_slice() {
+                [] => quote!(),
+                [..] => quote!(#(#attrs)*),
             };
             all_codes.push(quote! {
-                #attr Self::#linter(#linter::#prefix_ident)
+                #attrs Self::#linter(#linter::#prefix_ident)
             });
         }
 
@@ -159,12 +160,13 @@ pub(crate) fn map_codes(func: &ItemFn) -> syn::Result<TokenStream> {
                 quote!(#(#attrs)* Rule::#rule_name)
             });
             let prefix_ident = get_prefix_ident(&prefix);
-            let attr = match if_all_same(rules.iter().map(|(.., attrs)| attrs)) {
-                Some(attr) => quote!(#(#attr)*),
-                None => quote!(),
+            let attrs = intersection_all(rules.iter().map(|(.., attrs)| attrs.as_slice()));
+            let attrs = match attrs.as_slice() {
+                [] => quote!(),
+                [..] => quote!(#(#attrs)*),
             };
             prefix_into_iter_match_arms.extend(quote! {
-                #attr #linter::#prefix_ident => vec![#(#rule_paths,)*].into_iter(),
+                #attrs #linter::#prefix_ident => vec![#(#rule_paths,)*].into_iter(),
             });
         }
 
@@ -315,9 +317,16 @@ See also https://github.com/astral-sh/ruff/issues/2186.
                 matches!(self.group(), RuleGroup::Preview)
             }
 
-            #[allow(deprecated)]
-            pub fn is_nursery(&self) -> bool {
-                matches!(self.group(), RuleGroup::Nursery)
+            pub fn is_stable(&self) -> bool {
+                matches!(self.group(), RuleGroup::Stable)
+            }
+
+            pub fn is_deprecated(&self) -> bool {
+                matches!(self.group(), RuleGroup::Deprecated)
+            }
+
+            pub fn is_removed(&self) -> bool {
+                matches!(self.group(), RuleGroup::Removed)
             }
         }
 
@@ -359,13 +368,13 @@ fn generate_iter_impl(
 
     quote! {
         impl Linter {
-            /// Rules not in the nursery.
+            /// Rules not in the preview.
             pub fn rules(self: &Linter) -> ::std::vec::IntoIter<Rule> {
                 match self {
                     #linter_rules_match_arms
                 }
             }
-            /// All rules, including those in the nursery.
+            /// All rules, including those in the preview.
             pub fn all_rules(self: &Linter) -> ::std::vec::IntoIter<Rule> {
                 match self {
                     #linter_all_rules_match_arms
@@ -377,8 +386,10 @@ fn generate_iter_impl(
             pub fn iter() -> impl Iterator<Item = RuleCodePrefix> {
                 use strum::IntoEnumIterator;
 
-                std::iter::empty()
-                    #(.chain(#linter_idents::iter().map(|x| Self::#linter_idents(x))))*
+                let mut prefixes = Vec::new();
+
+                #(prefixes.extend(#linter_idents::iter().map(|x| Self::#linter_idents(x)));)*
+                prefixes.into_iter()
             }
         }
     }
@@ -465,7 +476,7 @@ fn register_rules<'a>(input: impl Iterator<Item = &'a Rule>) -> TokenStream {
 }
 
 impl Parse for Rule {
-    /// Parses a match arm such as `(Pycodestyle, "E112") => (RuleGroup::Nursery, rules::pycodestyle::rules::logical_lines::NoIndentedBlock),`
+    /// Parses a match arm such as `(Pycodestyle, "E112") => (RuleGroup::Preview, rules::pycodestyle::rules::logical_lines::NoIndentedBlock),`
     fn parse(input: syn::parse::ParseStream) -> syn::Result<Self> {
         let attrs = Attribute::parse_outer(input)?;
         let pat_tuple;
